/************************************************************/
/* unpure.c - read source for Pure-C helpfiles, remove all  */
/*            formatting instructions, and write to stdout. */
/* Version  0.2 (07/02/93) - add -subject option            */
/* Revision 0.1 (24/01/93) - first attempt                  */
/* Copyright (c) 1993 Hildo Biersma - Evil Eye Software     */
/*                    e-mail: boender@dutiws.twi.tudelft.nl */
/*                            biersma@xs4all.hacktic.nl     */
/* This program is Public Domain - do with it what you like */
/************************************************************/

/*
USAGE:
  unpure [-subject] libsource ...

  The libsource arguments contain the names of the *.src files
  to be searched. If the subject argument is missing, the whole
  contents will be printed. If the subject argument is present,
  any help screen with a name prefix matching subject will be
  printed. So, unpure -f *.src will print all screens with
  names starting with an f from all helpfiles.
*/

/*
BUGS:
  - will choke on strange characters and escapes in header
  - ditto in \links
  - will die spectacularly on input that would be invalid for HC
  - not written in ANSI-C to cover up for Sozobon's mistakes
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

/* Local defines */
#define SKIPPING  0
#define IN_HEADER 1
#define PRINTING  2
#define MAX_LINE  512

/* Local function prototypes */
#ifdef __STDC__
void process_file(char *filename);
#endif

/* Global variables */
char *program_name, *subject = NULL;
char line[MAX_LINE + 1], buffer[MAX_LINE + 1];
/* ^^ Assumption: every line will fit into MAX_LINE */

void main(argc, argv)
int argc;
char *argv[];
{
  int counter = 1;
  void process_file();

  program_name = argv[0];
  if (*argv[1] == '-')
  {
    subject = argv[1] + 1;
    counter = 2;
    argc -= 1;
  }

  while (argc > 1)
  {
    process_file(argv[counter]);
    counter += 1;
    argc -= 1;
  }
} /* End of main() */

/* Cat a single file to stdout, after removing formatting */
void process_file(filename)
char *filename;
{
  char *ptr1, *ptr2, *tmp;
  FILE *input;
  int found_subject = 0, end_subject = 0;

  int  state = SKIPPING, parens, quotes, linecount = 0;

  if ((input = fopen(filename, "r")) == NULL)
  {
    fprintf(stderr, "%s: couldn't open file %s\n",
            program_name, filename);
    return;
  }

  line[BUFSIZ] = 0x00;
  buffer[BUFSIZ] = 0x00;
  while (!feof(input))
  {
    fgets(line, MAX_LINE, input);
    ptr1 = line;
    ptr2 = buffer;
    switch(state)
    {
      case SKIPPING:
        if ((tmp = strstr(line, "screen")) == NULL)
          break;
        ptr1 = tmp + strlen("screen");
        state = IN_HEADER;
        parens = quotes = 0;
        /* FALL-THROUGH INTENDED */
      case IN_HEADER:
        while (*ptr1 != 0x00)
        {
          if (*ptr1 == '"')
          {
            if (quotes == 1)
              quotes = 0;
            else
            {
              quotes = 1;
              if ((subject != NULL) &&
                  (strncmp(subject, ptr1+1, strlen(subject)) == 0))
                found_subject = 1;
            }
            ptr1 += 1;
          }
          else if (*ptr1 == '(')
          {
            parens += 1;
            ptr1 += 1;
          }
          else if (*ptr1 == ')')
          {
            parens -= 1;
            if (parens == 0)
            {
              state = PRINTING;
              break;
            }
            ptr1 += 1;
          }
          else
            ptr1 += 1;
        }
        break;
      case PRINTING:
        while (*ptr1 != 0x00)
        {
          if (*ptr1 != '\\')
            *ptr2++ = *ptr1++;
          else
          {
            if (ptr1[1] == '\\')  /* \\ means \ */
            {
              *ptr2++ = '\\';
              ptr1 += 2;
            }
            else if (ptr1[1] == '#') /* Skip over \#...\# */
            {
              ptr1 += 2;
              while ((*ptr1 != '\\') && (ptr1[1] != '#'))
                *ptr2++ = *ptr1++;
              ptr1 += 2;
            }
            else if ((ptr1[1] == 'l') && /* Skip over \link("...")...\# */
                     (ptr1[2] == 'i') &&
                     (ptr1[3] == 'n') &&
                     (ptr1[4] == 'k'))
            {
              ptr1 += 7; /* Skip over \link(" */
              while ((*ptr1 != '"') && (ptr1[1] != ')'))
                ptr1++;
              ptr1 += 2;
              while ((*ptr1 != '\\') && (ptr1[1] != '#'))
                *ptr2++ = *ptr1++;
              ptr1 += 2;
            }
            else if ((ptr1[1] == 'e') && /* End page on \end */
                     (ptr1[2] == 'n') &&
                     (ptr1[3] == 'd'))
            {
              *ptr2++ = '\n';
              *ptr2++ = 0x0c; /* Page feed */
              *ptr1 = 0x00;
              state = SKIPPING;
              end_subject = 1;
            }
            else
            {
              fprintf(stderr, "%s: choked on line: %s\n",
                      program_name, line);
              exit(1);
            }
          }
        } /* End of while (line not copied) */
        *ptr2 = 0x00;
        linecount++;
        if ((subject == NULL) || (found_subject == 1))
          fputs(buffer, stdout);
        if (end_subject == 1)
        {
          end_subject = 0;
          found_subject = 0;
        }
        break;
      default:
        fprintf(stderr, "%s: internal or compiler error\n",
                program_name);
        exit(1);
    } /* End of switch(state) */
  }

  fclose(input);
} /* End of process_file() */
